 // ----------------------------- SMS Relay Controller -----------------------------------------------------------------------------------------------

// Designed for use with Haute Solutions MEGGABOARD3G or ESP324G Boards and the Haute Solutions OLED Display
// www.haute-solutions.com

// Haute Solutions Libraries Required: SMSDEV ( GSMSMS, OLED_SSD1311 )

// --------------------------------------------------------------------------------------------------------------------------
#include "SMSDEV.h"
#include "PCF8574.h"

#define COMPANY   "Haute Solutions"
#define DEVICE    "SMS Relay"
#define VERSION   "V7.1"
#define EEPROMSIG "SMSRELAY EEPROM V3"    // EEPROM Valid Signature (19 Char Max)

//#define THREEG                            // Compile for MeGGaBoard3G (Mega2560)
#define FOURG                               // Compile for ESP324G (Esp32)
#define OLEDADDR 0x3C                       // I2C Address for OLED Display (NULL if None)
#define PCFADDR  0x38                       // I2C Address for PCF8574 Buttons

#ifdef THREEG
#define BOARD MEGGABOARD3G
#define RELAY_CTRL_PIN 9
#define MANUAL_SW_PIN 8
// SDA = 20
// SCL = 21
#endif

#ifdef FOURG
#define BOARD ESP324G
#define RELAY_CTRL_PIN 25
#define MANUAL_SW_PIN 16
// SDA = 21
// SCL = 22
#endif

const CMDINFO Commands[] = {                    // Device Specific Commands (Generic SMS Commands are defined in SMSDEV Library)
  {"HELP",         1, 0},
  {"STATUS",       2, 0},
  {"ON",           3, 0},
  {"OFF",          4, 0}
};

// Define some global shorthand "aliases" (for readability)
char* Buffer;
char* Caller;
char* Temp;

SMSDEV SMSDev(BOARD, OLEDADDR);
PCF8574 pcf(PCFADDR);

void setup(){
int rcode;

  Serial.begin(9600);

  pcf.begin();

  SetButtons();

  rcode = SMSDev.Setup(COMPANY, DEVICE, VERSION, EEPROMSIG, NULL, Commands, sizeof(Commands)/sizeof(Commands[0]));
  while (rcode > 0) { }
  
  // Assign our Global Aliases
  Buffer       = SMSDev.Buffer;
  Caller       = SMSDev.Caller;
  Temp         = SMSDev.Temp;

  DisplaySchedule();

  SMSDev.DisplayAuthCallers();

  if(SMSDev.GetAuthCaller(0, Caller) != 0) {
    if(SMSDev.BootNotify[0] == '1') {
      CommandSTATUS(Caller, "System Ready!");
      delay(2000);
      SMSDev.SendAuthList(Caller);
    }
  }

  SMSDev.DisplayIdleScreen();
}

void loop(){
    
  switch (SMSDev.Loop()) {
    case -9:  // Factory Reset (SMSDEV)
      CommandRESET();
      break;

    case 1:   // HELP
      CommandHELP();
      break;
    case 2:   // STATUS
      CommandSTATUS(Caller, NULL);
      break;
    case 3:   // ON
      CommandON();
      break;
    case 4:   // OFF
      CommandOFF();
      break;

    case 100: // Clock Tick (every 30 sec)
      // Nothing Required
      break;
    case 101: // Alarm 1 (ON)
      SMSDev.ReadEEPROMField(EEDEVSPEC1, Temp);
      TurnOn(Temp);
      break;
    case 102: // Alarm 2 (OFF)
      SMSDev.ReadEEPROMField(EEDEVSPEC2, Temp);
      TurnOff(Temp);
      break;
    case 103: // Status Request (HeartBeat)
      if(SMSDev.GetAuthCaller(0, Caller) != 0) CommandSTATUS(Caller, "HeartBeat!");
      break;
    case 104: // CSQ (Signal Change)
      // Display Updated in SMSDEV
      break;
    case 200: // Unauthorized Caller 
      // Message Forwarded in SMSDEV
      break;
    case 201: // Invalid Command
      // Reply Handled in SMSDEV
      break;
    case 202: // Command Restricted
      // Reply Handled in SMSDEV
      break;
    case 203: // Forwarded Message
      // Reply Handled in SMSDEV
      break;
  }

  CheckButtons();
}

// ---------------------------- Button Functions ---------------------------------------

void SetButtons() {
  
  // Relay Control Pin
  pinMode(RELAY_CTRL_PIN, OUTPUT);
  
  // Manual Swiches
  pinMode(MANUAL_SW_PIN, INPUT);

}

void CheckButtons() {
byte Buttons;

  Buttons = pcf.read8();
  for(int i=0; i<4; i++) {
    if((Buttons & (1 << i)) == 0) {
      switch (i) {
        case 0:
          SMSDev.DisplayIdleScreen();
          return;
        case 1:
          DisplaySchedule();
          SMSDev.DisplayTime();
          SMSDev.DisplayIdleScreen();
          return;
        case 2:
          SMSDev.DisplayAuthCallers();
          SMSDev.DisplayIdleScreen();
          return;
        case 3:
          if(DisplayCountdown(i, "Factory Reset")) {
            SMSDev.FactoryReset();
            CommandRESET();
          }
          return;
      }
    }   
  }
  
  if(digitalRead(MANUAL_SW_PIN) == HIGH) {
    delay(500); // Debounce
    if(digitalRead(RELAY_CTRL_PIN) == LOW) {
      digitalWrite(RELAY_CTRL_PIN, HIGH);
      SMSDev.Display("Manual Control", "ON", 1);
    } else {
      digitalWrite(RELAY_CTRL_PIN, LOW);
      SMSDev.Display("Manual Control", "OFF", 1);
    }
    delay(MSGDISPLAYTIME);
    SMSDev.DisplayIdleScreen();
  }
}

int DisplayCountdown(int Button, const char* Msg) {
unsigned long ResetTimer;
char ALine[20];
int SCheck;
int SecsRemaining = -1;
int SecsToWait = 5;

  ResetTimer = millis();
  do {
    SCheck = SecsToWait - ((millis() - ResetTimer) / 1000);
    if(SecsRemaining != SCheck) {
      SecsRemaining = SCheck;
      sprintf(ALine,"in %d Seconds...", SecsRemaining);
      SMSDev.Display(Msg, ALine);
    }
    if(SecsRemaining == 0) return 1;
  } while ((pcf.read8() & (1 << Button)) == 0);  

  SMSDev.DisplayIdleScreen();

  return 0;
}

// -------------------------------------------------------------------------------------

void DisplaySchedule() {
  
  if(strlen(SMSDev.Alarm1)) {
    SMSDev.Display("On at:", SMSDev.Alarm1);
    delay(2000);
    SMSDev.ReadEEPROMField(EEDEVSPEC1, Temp);
    SMSDev.Display("Scheduled By", Temp);
  } else SMSDev.Display("On at:","(Not Scheduled)");
  delay(MSGDISPLAYTIME);

  if(strlen(SMSDev.Alarm2)) {
    SMSDev.Display("Off at:", SMSDev.Alarm2);
    delay(2000);
    SMSDev.ReadEEPROMField(EEDEVSPEC2, Temp);
    SMSDev.Display("Scheduled By", Temp);
  } else SMSDev.Display("Off at:","(Not Scheduled)");
  delay(MSGDISPLAYTIME);

  SMSDev.DisplayIdleScreen();
}

// Command Processing Functions -----------------------------------------------------------------------------

void CommandRESET() {

  // Majority of Factory Reset handled by SMSDEV (inc Notification)
  SMSDev.WriteEEPROMField(EEDEVSPEC1, "");
  SMSDev.WriteEEPROMField(EEDEVSPEC2, "");
}

void CommandHELP() {

  strcpy(Buffer, DEVICE);
  strcat(Buffer, " Commands:\n\n");
  
  strcat(Buffer, "ON\n");
  strcat(Buffer, "OFF\n");
  strcat(Buffer, "ON @ YY/MM/DD,HH:SS\n");
  strcat(Buffer, "OFF @ YY/MM/DD,HH:SS\n");
  strcat(Buffer, "STATUS\n");

  SMSDev.SendSMSWithDisplay(Caller,Buffer);  
}

int CommandSTATUS(const char* acaller, const char *msg) {
int  i;
int  sigstars;
char msgend[2]={0x1a,'\0'};
char csqval[2];
char EEData[EEFIELDSIZE];

  //  We are going to build our SMS Message incrementally using the low level library functions
  //  So we don't have to allocate a bunch of memory for a temp string

  if(SMSDev.PrepToSendSMS(acaller) > 0) return 1;
  
  if(msg != NULL) {
    SMSDev.Print(msg);
    SMSDev.Print("\n\n");
  }

  SMSDev.Print("SMS Relay: ");
  SMSDev.Print(VERSION);
  SMSDev.Print("\n");
    
  SMSDev.Print("Time: ");
  SMSDev.Print(SMSDev.TimeNow);
  SMSDev.Print("\n");
    
  SMSDev.Print("Status: ");
  if(digitalRead(RELAY_CTRL_PIN) == HIGH) SMSDev.Print("ON");
  else SMSDev.Print("OFF");
  SMSDev.Print("\n");

  SMSDev.Print("On @ ");
  if(strlen(SMSDev.Alarm1) == 0) {
    SMSDev.Print("(Not Set)");
  } else {
    SMSDev.Print(SMSDev.Alarm1);
  }
  SMSDev.Print("\n");
  
  SMSDev.Print("Off @ ");
  if(strlen(SMSDev.Alarm2) == 0) {
    SMSDev.Print("(Not Set)");
  } else {
    SMSDev.Print(SMSDev.Alarm2);
  }
  SMSDev.Print("\n");

  csqval[0]=char(SMSDev.CSQ + 48);
  csqval[1]='\0';
  SMSDev.Print("Signal: ");
  SMSDev.Print(csqval);
  SMSDev.Print("/5");
  SMSDev.Print("\n");
   
  SMSDev.IndicateSMSOut(acaller);

  SMSDev.Print(msgend);
  if(SMSDev.FlushSMS() > 0) return 2;

  return 0;
}

void CommandON() {
char* p1;

    p1 = strchr(Buffer, '@');
    if(p1 != NULL) {
      p1 = p1 + strspn(p1, "@ ");
      strncpy(SMSDev.Alarm1, p1, 14);
      SMSDev.Alarm1[14]='\0';
      SMSDev.WriteEEPROMField(EEALARM1, SMSDev.Alarm1);
      SMSDev.WriteEEPROMField(EEDEVSPEC1, Caller);
      CommandSTATUS(Caller, "Scheduled!");
    } else TurnOn(Caller);
    return;
}

void TurnOn(const char* acaller) {
  digitalWrite(RELAY_CTRL_PIN, HIGH);
  CommandSTATUS(acaller, "ON!");  
}

void CommandOFF() {
char* p1;

    p1 = strchr(Buffer, '@');
    if(p1 != NULL) {
      p1 = p1 + strspn(p1, "@ ");
      strncpy(SMSDev.Alarm2, p1, 14);
      SMSDev.Alarm2[14]='\0';
      SMSDev.WriteEEPROMField(EEALARM2, SMSDev.Alarm2);
      SMSDev.WriteEEPROMField(EEDEVSPEC2, Caller);
      CommandSTATUS(Caller, "Scheduled!");
    } else TurnOff(Caller);

    return;
}

void TurnOff(const char* acaller) {
  digitalWrite(RELAY_CTRL_PIN, LOW);
  CommandSTATUS(acaller, "OFF!");  
}
